<?php

/**
 * @file
 * Contains \Drupal\miniorange_oauth_client\Form\MiniorangeConfigOAuthClient.
 */
namespace Drupal\miniorange_oauth_client\Form;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\FormBase;
use Drupal\miniorange_oauth_client\Utilities;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Core\Url;
use Drupal\Core\Render\Markup;
use Symfony\Component\HttpFoundation\Response;

class MiniorangeConfigOAuthClient extends FormBase
{
    public function getFormId() {
       return 'miniorange_oauth_client_settings';
    }

    public function buildForm(array $form, FormStateInterface $form_state)
    {
        global $base_url;
        \Drupal::configFactory()->getEditable('miniorange_oauth_client.settings')->set('miniorange_oauth_client_disabled', FALSE)->save();
        $baseUrl = \Drupal::config('miniorange_oauth_client.settings')->get('miniorange_oauth_client_base_url');
        $baseUrlValue = empty($baseUrl) ? Utilities::getOAuthBaseURL($base_url)  : $baseUrl ;
        $attachments['#attached']['library'][] = 'miniorange_oauth_client/miniorange_oauth_client.admin';
        $form['markup_library'] = array(
            '#attached' => array(
               'library' => array(
                    "miniorange_oauth_client/miniorange_oauth_client.oauth_config",
                    "miniorange_oauth_client/miniorange_oauth_client.admin",
                    "miniorange_oauth_client/miniorange_oauth_client.testconfig",
                    "miniorange_oauth_client/miniorange_oauth_client.returnAttribute",
                    "miniorange_oauth_client/miniorange_oauth_client.style_settings",
                    "miniorange_oauth_client/miniorange_oauth_client.Vtour",
               )
            ),
        );

        $app_name = \Drupal::config('miniorange_oauth_client.settings')->get('miniorange_auth_client_app_name');
        $client_id = \Drupal::config('miniorange_oauth_client.settings')->get('miniorange_auth_client_client_id');
        if(!empty($app_name) && !empty($client_id)){
            $disabled = TRUE;
        }

        $disableButton = NULL;
        if( empty($app_name)  || empty($client_id) ){
            $disableButton = 'disabled';
        }

        $app_name_selected = \Drupal::config('miniorange_oauth_client.settings')->get('miniorange_auth_client_app_name');
        $client_id = \Drupal::config('miniorange_oauth_client.settings')->get('miniorange_auth_client_client_id');

        if(isset($_GET['action']) && $_GET['action'] == 'update' ){

            if(!empty($client_app) && !empty($client_id)){
                $disabled = TRUE;
            }
            else{
                $disabled = FALSE;
            }

            $form['mo_oauth_top_div'] = array('#markup' => '<div class="mo_oauth_table_layout_1">');

            $form['mo_oauth_inside_div'] = array(
              '#markup' => '<div class="mo_oauth_table_layout mo_oauth_container">',
            );

            $form['miniorange_oauth_client_summary'] = array(
                '#type' => 'table',
                '#responsive' => TRUE ,
                '#caption' => $this->t('<h3>APPLICATION DETAILS</h3><hr>'),
                '#attributes' => ['style' => 'border-collapse: separate;','class' => ['configtable'],],
            );

            $form['#attached']['library'][] = 'core/drupal.dialog.ajax';

            $configurations = self::get_mo_oauth_configurations();


            foreach($configurations as $key => $value){
                $row = self::generate_mo_oauth_configuration_table_row($key, $value);
                $form['miniorange_oauth_client_summary'][$key] = $row;
            }

            $form['miniorange_oauth_client_config_submit'] = array(
                '#type' => 'submit',
                '#id' => 'save_button',
                '#value' => t('Save Configuration'),
                '#button_type' => 'primary',
                '#prefix' => '<div class="oauth_config_buttons">'
            );

            $baseUrlValue = Utilities::getOAuthBaseURL($base_url);

            $form['mo_reset'] = array(
                '#markup' => "<a class='button button--danger' id ='vt_reset_config' ".$disableButton." href='".$baseUrlValue."/resetConfig'>Reset Configuration</a></div>",
            );

            $form['miniorange_oauth_login_link'] = array(
                '#id'  => 'miniorange_oauth_login_link',
                '#markup' => "<br><br><div class='mo_oauth_instruction_style'>
                    <br><strong><div class='mo_custom_font_size_1'>Instructions to add login link to different pages in your Drupal site: </div></strong><br>
                    <div class='mo_custom_font_size_2'>After completing your configurations, by default you will see a login link on your drupal site's login page.
                    However, if you want to add login link somewhere else, please follow the below given steps:</div>
                    <div class='mo_custom_font_size_3'>
                    <li>Go to <b>Structure</b> -> <b>Blocks</b></li>
                    <li> Click on <b>Add block</b></li>
                    <li>Enter <b>Block Title</b> and the <b>Block description</b></li>
                    <li>Under the <b>Block body</b> add the following URL to add a login link:
                        <ol> <h6>&lt;a href= '".$baseUrlValue."/moLogin'> Click here to Login&lt;/a&gt;</h6></ol>
                    </li>
                    <li>From the text filtered dropdown select either <b>Filtered HTML</b> or <b>Full HTML</b></li>
                    <li>From the division under <b>REGION SETTINGS</b> select where do you want to show the login link</li>
                    <li>Click on the <b>SAVE block</b> button to save your settings</li><br>
                    </div>
                    </div>",
                '#attributes' => array(),
            );

            $form['mo_header_style_end'] = array('#markup' => '</div></div>');

            Utilities::moOAuthShowCustomerSupportIcon($form, $form_state);
            return $form;
        }else if((isset($_GET['action']) && $_GET['action'] == 'delete' )){

            $config = \Drupal::configFactory()->getEditable('miniorange_oauth_client.settings');
            $config  ->clear('miniorange_oauth_enable_login_with_oauth')
                    ->clear('miniorange_oauth_client_app')
                    ->clear('miniorange_oauth_client_appval')
                    ->clear('miniorange_auth_client_app_name')
                    ->clear('miniorange_auth_client_display_name')
                    ->clear('miniorange_auth_client_client_id')
                    ->clear('miniorange_auth_client_client_secret')
                    ->clear('miniorange_auth_client_scope')
                    ->clear('miniorange_auth_client_authorize_endpoint')
                    ->clear('miniorange_auth_client_access_token_ep')
                    ->clear('miniorange_auth_client_user_info_ep')
                    ->clear('miniorange_auth_client_stat')
                    ->clear('miniorange_auth_client_callback_uri')
                    ->clear('miniorange_oauth_send_with_header_oauth')
                    ->clear('miniorange_oauth_client_attr_list_from_server')
                    ->set('miniorange_oauth_send_with_body_oauth',1)
                    ->save();

            $form['mo_back'] = array(
            '#markup' => "
                    <br><h2>Configurations deleted successfully</h2><br><a class='btn btn-primary btn-large button button--primary' style='padding:10px 22px;' href='config_clc'>Back</a>",
            );

            \Drupal::messenger()->addMessage(t('Application Removed Successfully.'));
            return $form;
        }

        $client_app = \Drupal::config('miniorange_oauth_client.settings')->get('miniorange_oauth_client_app');

        if($client_app != NULL)
        {
            $form['#attached']['library'][] = 'core/drupal.dialog.ajax';

            $form['mo_oauth_top_div'] = array('#markup' => '<div class="mo_oauth_table_layout_1">');

            $form['mo_style_header_start'] = array('#markup' => '<div class="mo_oauth_table_layout mo_oauth_container_center">',);

            $form['miniorange_oauth_client_msgs'] = array(
                '#markup' => "<div class='mo_oauth_highlight_background_note_add_new_provider'>
                           <b class='mo_note_css'>Please Note:</b> Attribute Mapping is mandatory for login. Select the Email Attribute from Test Configuration and click on the <b>Done</b> button.</div><br>",
            );

            $form['miniorange_oauth_client'] = array(
                '#markup' => '<a data-dialog-type="modal" href="addnewprovider" class="use-ajax button button--primary add_new_provider">+Add New Provider</a>'
            );

            $header = array(
                'idp_name' => array(
                  'data' => t('Provider Name')
                ),
                'client_id' => array(
                    'data' => t('Client ID')
                ),
                'test' => array(
                  'data' => t('Test')
                ),
                'action' => array(
                  'data' => t('Action')
                ),
                'mapping' => array(
                  'data' => t('Attribute & Role Mapping')
                ),
              );


                $drop_button = array(
                  '#type' => 'dropbutton',
                  '#dropbutton_type' => 'small',
                  '#links' => [
                    'edit' => [
                      'title' => $this->t('Edit'),
                      'url' => Url::fromUri($base_url . '/admin/config/people/miniorange_oauth_client/config_clc?action=update&app=' . $client_app),
                    ],
                    'delete' => [
                      'title' => $this->t('Delete'),
                      'url' => Url::fromUri($base_url . '/admin/config/people/miniorange_oauth_client/config_clc?action=delete&app=' . $client_app),
                    ],

                    'Backup/Import' => [
                        'title' => $this->t('Backup/Import'),
                        'url' => Url::fromUri($base_url.'/admin/config/people/miniorange_oauth_client/backup'),
                        'localized_options' => [
                            'attributes' => [
                               'class' => ['use-ajax']
                            ],
                    ],
                  ],
                ]
                );

                $client_id = \Drupal::config('miniorange_oauth_client.settings')->get('miniorange_auth_client_client_id');
                $client_id = strlen($client_id) > 25 ? substr($client_id,0,24).'...' : $client_id;

                $table_content['rows'] = array(
                  'idp_name' => $client_app,
                  'client_id' => $client_id,
                  'test' => Markup::create('<span id="base_Url" name="base_Url" data='. $baseUrlValue.'></span><a class="button button--primary button--small " href="#" id="testConfigButton">Perform Test Configuration</a>'),
                  'action' => array(
                    'data' => $drop_button,
                  ),
                  'mapping' => Markup::create('<a class="button button--small" href="mapping">Configure</a>'),
                );

                $form['mo_oauth_client_idplist_table'] = array(
                  '#type' => 'table',
                  '#header' => $header,
                  '#rows' => $table_content,
                  '#empty' => t('<b>You have not configured any provider yet, Please Add provider by clicking above "Add New SP" button</b>'),
                  '#prefix' => '<br><br><br>',
                  '#suffix' => '</div>',
                  '#disabled' => $disabled,
                  '#attributes' => array(
                    'class' => array('tableborder'),),
                );

            $form['div2_close'] =array(
                '#markup' => '</div></div>'
            );
            Utilities::moOAuthShowCustomerSupportIcon($form,$form_state);
            return $form;
        }

        if(!empty($app_name_selected) && !empty($client_id)){
            $disabled = TRUE;
        }
        else{
            $disabled = FALSE;
        }

        $form['mo_oauth_top_div'] = array('#markup' => '<div class="mo_oauth_table_layout_1">');

        $form['mo_oauth_inside_div'] = array(
          '#markup' => '<div class="mo_oauth_table_layout mo_oauth_container">',
        );

        $form['miniorange_oauth_client_summary'] = array(
            '#type' => 'table',
            '#responsive' => TRUE ,
            '#caption' => $this->t('<span><h3>CONFIGURE APPLICATION</h3><div class ="container-inline" id="setupguide"></div><hr></span>'),
            '#attributes' => ['style' => 'border-collapse: separate;','class' => ['configtable'],],
        );

        $form['#attached']['library'][] = 'core/drupal.dialog.ajax';

        $configurations =  self::get_mo_oauth_configurations();
        $configurations['miniorange_oauth_client_app'] = 'Select Application';

        foreach($configurations as $key => $value){
            $row = self::generate_mo_oauth_configuration_table_row($key, $value);
            $form['miniorange_oauth_client_summary'][$key] = $row;
        }

        $form['miniorange_oauth_client_config_submit'] = array(
            '#type' => 'submit',
            '#id' => 'save_button',
            '#value' => t('Save Configuration'),
            '#button_type' => 'primary',
            '#prefix' => '<div class="oauth_config_buttons">',
             '#suffix'=> '</div>'
        );

        $baseUrlValue = Utilities::getOAuthBaseURL($base_url);

        $form['miniorange_oauth_login_link'] = array(
            '#id'  => 'miniorange_oauth_login_link',
            '#markup' => "<br><br><div class='mo_oauth_instruction_style'>
                <br><strong><div class='mo_custom_font_size_1'>Instructions to add login link to different pages in your Drupal site: </div></strong><br>
                <div class='mo_custom_font_size_2'>After completing your configurations, by default you will see a login link on your drupal site's login page.
                However, if you want to add login link somewhere else, please follow the below given steps:</div>
                <div class='mo_custom_font_size_3'>
                <li>Go to <b>Structure</b> -> <b>Blocks</b></li>
                <li> Click on <b>Add block</b></li>
                <li>Enter <b>Block Title</b> and the <b>Block description</b></li>
                <li>Under the <b>Block body</b> add the following URL to add a login link:
                    <ol> <h6>&lt;a href= '".$baseUrlValue."/moLogin'> Click here to Login&lt;/a&gt;</h6></ol>
                </li>
                <li>From the text filtered dropdown select either <b>Filtered HTML</b> or <b>Full HTML</b></li>
                <li>From the division under <b>REGION SETTINGS</b> select where do you want to show the login link</li>
                <li>Click on the <b>SAVE block</b> button to save your settings</li><br>
                </div>
                </div>",
            '#attributes' => array(),
        );

        $form['mo_header_style_end'] = array('#markup' => '</div></div></div>');

        Utilities::moOAuthShowCustomerSupportIcon($form, $form_state);
        return $form;
    }

    public static function generate_mo_oauth_configuration_table_row($key, $value){

        global $base_url;
        $config = \Drupal::config('miniorange_oauth_client.settings');
        $module_path = \Drupal::service('extension.list.module')->getPath('miniorange_oauth_client');

        $description = self::get_description_for_configuration();

        if($key == 'send_client_id_secret'){
            $send_credentials = $value;
            $value = 'Send Client ID and secret in: <div class="mo_oauth_tooltip"><img src="'.$base_url.'/'. $module_path . '/includes/images/info.png" alt="info icon" height="15px" width="15px"></div><div class="mo_oauth_tooltiptext"><b>Note:</b> This option depends upon the OAuth provider. In case you are unaware about what to save, keeping this default is the best practice.</div>';
        }

        if($value == 'Callback/Redirect URL'){
            $value = 'Callback/Redirect URL <div class="mo_oauth_tooltip_cb"><img src="'.$base_url.'/'. $module_path . '/includes/images/info.png" alt="info icon" height="15px" width="15px"></div><div class="mo_oauth_tooltiptext_cb"><b>Note:</b> If your provider only support HTTPS <b>Callback/Redirect URL </b>and you have HTTP site, just save your base site URL with HTTPS in the <b>Sign In Settings</b> tab.
            </div>';
        }

        if($value == 'Enable Login with OAuth' || $key == 'miniorange_auth_client_callback_uri' || $key == 'grant_type_selected' || $value == 'Login link on the login page' || $key == 'send_client_id_secret'){
            $required = false;
            $row[$key.$value] = [
                '#markup' => '<div class="container-inline mo-table_app1"><strong>'.$value.'</strong>',
            ];
        }else{
            $required = true;
            $row[$key.$value] = [
                '#markup' => '<div class="container-inline mo-table_app1"><strong>'.$value.'<span class="mo_note_css">*<span></strong>',
            ];
        }

        if($value == 'Enable Login with OAuth'){
            $row[$key] = [
                '#type' => 'checkbox',
                '#default_value' => $config->get($key),
                '#title' => t('<i>( Note: Check this option to show SSO link on the Login page) </i>'),
            ];

        }else if($key == 'miniorange_auth_client_callback_uri'){
            $row[$key][$key] = array(
                '#markup' => '<span id="callbackurl">'.$config->get($key).'&nbsp;</span>',
                '#prefix' => '<span class="container-inline">',

            );

            $row[$key]['miniorange_oauth_copy'] = array(
                '#markup' => '<div class="callback_tooltip">
                <span class="button mo_copy_url button--small" id="copy_button">
                  <span class="tooltiptext" id="myTooltip">Copy to Clipboard</span>
                  &#128461; Copy
                  </span>
                </div></span>',
            );

        }else if($value == 'Application Name'){
            $row[$key] = array(
                '#type' => 'textfield',
                '#id' => $key,
                '#disabled' => true,
                '#default_value' => $config->get($key),
            );

        }else if($key == 'grant_type_selected'){
            $row[$key] = array(
                '#type' => 'radios',
                '#options' => array('authorization_code' => 'Authorization Code Grant &nbsp;<a class="use-ajax" data-dialog-type="modal" href="authorizationcodegrant"><div class ="mo_oauth_tooltip"><img class = "question" src="'. $base_url . '/' . $module_path . '/includes/images/questionmark-disc.svg" width="18"></div></a>&nbsp;','authorization_code_with_pkce' => 'Authorization Code with PKCE &nbsp;<a class="use-ajax" data-dialog-type="modal" href="authorizationcodewithpkcegrant"><div class ="mo_oauth_tooltip"><img class = "question" src="'. $base_url . '/' . $module_path . '/includes/images/questionmark-disc.svg" width="18"></div></a>&nbsp;', 'password' => 'Password Grant &nbsp;<a class="use-ajax" data-dialog-type="modal" href="passwordgrant"><div class ="mo_oauth_tooltip"><img class = "questio" src="'. $base_url . '/' . $module_path . '/includes/images/questionmark-disc.svg" width="18"></div></a>&nbsp;', 'implicit' => 'Implicit Grant &nbsp;<a class="use-ajax" data-dialog-type="modal" href="implicitgrant"><div class ="mo_oauth_tooltip"><img class = "questio" src="'. $base_url . '/' . $module_path . '/includes/images/questionmark-disc.svg" width="18"></div></a>'),
                '#default_value' => 'authorization_code',
                '#disabled' => TRUE,
                '#prefix' => '<span class="container-inline client_cred">',
                '#suffix' => '</span>',
                '#attributes' => array(
                    'class' => array('container-inline'),),
            );
        }else if($key == 'send_client_id_secret'){
            $row[$key.'_oauth'][$send_credentials[0]] = array(
                '#type' => 'checkbox',
                '#title' => '&nbsp;Header',
                '#default_value' => $config->get($send_credentials[0]),
                '#prefix' => '<span class="container-inline client_cred">'
            );

            $row[$key.'_oauth'][$send_credentials[1]] = array(
                '#type' => 'checkbox',
                '#title' => '&nbsp;&nbsp;&nbsp;Body',
                '#attributes' => array(
                    'class' => array('client_cred'),),
                '#attributes' => ['style' => 'margin-left: 30px;'],
                '#default_value' => $config->get($send_credentials[1]),
                '#suffix' => '</span>'
            );

        }else if($key == 'miniorange_oauth_client_app'){
            $row[$key] = array(
                '#type' => 'select',
                '#options' => self::get_oauth_providers(),
                '#id' => $key,
                '#required' => true,
                '#description' => t('Select an OAuth Server')
            );
        }else{
            $row[$key] = array(
              '#type' => 'textfield',
              '#id' => $key,
              '#default_value' => $config->get($key),
              '#required' => $required,
              '#description' => isset($description[$key]) ? $description[$key] : ''
            );
        }

        return $row;

    }

    public static function get_oauth_providers(){
        $oauth_providers = [
            'Azure AD' => t('Azure AD'),
            'Box' => t('Box'),
            'Discord' => t('Discord'),
            'Facebook' => t('Facebook'),
            'FitBit' => t('FitBit'),
            'GitHub' => t('GitHub'),
            'Google' => t('Google'),
            'Keycloak' => t('Keycloak'),
            'Line' => t('Line'),
            'LinkedIn' => t('LinkedIn'),
            'Okta' => t('Okta (OAuth)'),
            'Paypal' => t('Paypal'),
            'Salesforce' => t('Salesforce'),
            'Slack' => t('Slack'),
            'Strava' => t('Strava'),
            'Wild Apricot' => t('Wild Apricot'),
            'Zendesk' => t('Zendesk'),
            'Custom' => t('Custom OAuth 2.0 Provider'),
            'Azure AD B2C' => t('Azure AD B2C (Premium and Enterprise)'),
            'AWS Cognito' => t('AWS Cognito (Premium and Enterprise)'),
            'Onelogin' => t('Onelogin (Premium and Enterprise)'),
            'miniOrange' => t('miniOrange (Premium and Enterprise)'),
            'Okta_openid' => t('Okta (OpenID) (Premium and Enterprise)'),
            'Custom_openid' => t('Custom OpenID Provider (We support OpenID protocol in Premium and Enterprise version)')
        ];
        return $oauth_providers;
    }

    public static function get_mo_oauth_configurations(){

        global $base_url;
        $url_path = $base_url . '/' . \Drupal::service('extension.list.module')->getPath('miniorange_oauth_client'). '/includes/images';

        $OAuth_configuration =  array(
            'miniorange_oauth_client_app' => 'Application Name',
            'miniorange_auth_client_callback_uri' => 'Callback/Redirect URL',
            'miniorange_auth_client_app_name' => 'Display Name',
            'miniorange_auth_client_display_name' => 'Login link on the login page',
            'grant_type_selected' => 'Grant Types<a href="licensing"><img src="' . $url_path . '/pro.png" alt="ENTERPRISE" class="shortened-image"><span class="mo_pro_tooltip">Available in the Enterprise version</span></a>',
            'miniorange_auth_client_client_id' => 'Client ID',
            'miniorange_auth_client_client_secret' => 'Client Secret',
            'miniorange_auth_client_scope' => 'Scope',
            'miniorange_auth_client_authorize_endpoint' => 'Authorize Endpoint',
            'miniorange_auth_client_access_token_ep' => 'Access Token Endpoint',
            'miniorange_auth_client_user_info_ep' => 'Get User Info Endpoint',
            'send_client_id_secret' => ['miniorange_oauth_send_with_header_oauth','miniorange_oauth_send_with_body_oauth'],
            'miniorange_oauth_enable_login_with_oauth' => 'Enable Login with OAuth',
        );

        return $OAuth_configuration;
    }

    public static function get_description_for_configuration(){
        $oauth_desc= array();

        $oauth_desc = array(
            'miniorange_auth_client_display_name' => t('<b>Note:</b> The login link will appear on the user login page in this manner'),
            'miniorange_auth_client_scope' => t('Scope decides the range of data that you will be getting from your OAuth Provider'),
        );

        return $oauth_desc;
    }

    public function validateForm(array &$form, FormStateInterface $form_state) {
        $form_values = $form_state->getValues();
        $enable_with_header = $form_values['miniorange_oauth_client_summary']['send_client_id_secret']['send_client_id_secret_oauth']['miniorange_oauth_send_with_header_oauth'];
        $enable_with_body = $form_values['miniorange_oauth_client_summary']['send_client_id_secret']['send_client_id_secret_oauth']['miniorange_oauth_send_with_body_oauth'];

        if ($enable_with_header == 0 && $enable_with_body == 0 ) {
            $form_state->setErrorByName('miniorange_oauth_client', t('This state is not allowed. Please select at least one of the options to send Client ID and Secret.'));
        }
    }

    public function submitForm(array &$form, FormStateInterface $form_state) {
        global $base_url;
        $baseUrlValue = Utilities::getOAuthBaseURL($base_url);
        $form_values = $form_state->getValues();

        if(isset($form_values['miniorange_oauth_client_summary']['miniorange_oauth_client_app']))
            $client_app =  trim($form_values['miniorange_oauth_client_summary']['miniorange_oauth_client_app']['miniorange_oauth_client_app']);
        if(isset($form_values['miniorange_oauth_client_summary']['miniorange_auth_client_app_name']))
            $app_name = trim( $form_values['miniorange_oauth_client_summary']['miniorange_auth_client_app_name']['miniorange_auth_client_app_name'] );
        $app_name = str_replace(' ', '', $app_name);

        if(isset($form_values['miniorange_oauth_client_summary']['miniorange_auth_client_display_name']))
            $display_name = trim( $form_values['miniorange_oauth_client_summary']['miniorange_auth_client_display_name']['miniorange_auth_client_display_name']);
        if(isset($form_values['miniorange_oauth_client_summary']['miniorange_auth_client_client_id']))
            $client_id = trim( $form_values['miniorange_oauth_client_summary']['miniorange_auth_client_client_id']['miniorange_auth_client_client_id'] );
        if(isset($form_values['miniorange_oauth_client_summary']['miniorange_auth_client_client_secret']))
            $client_secret = trim( $form_values['miniorange_oauth_client_summary']['miniorange_auth_client_client_secret']['miniorange_auth_client_client_secret']);
        if(isset($form_values['miniorange_oauth_client_summary']['miniorange_auth_client_scope']))
            $scope = trim( $form_values['miniorange_oauth_client_summary']['miniorange_auth_client_scope']['miniorange_auth_client_scope']);
        if(isset($form_values['miniorange_oauth_client_summary']['miniorange_auth_client_authorize_endpoint']))
            $authorize_endpoint = trim($form_values['miniorange_oauth_client_summary']['miniorange_auth_client_authorize_endpoint']['miniorange_auth_client_authorize_endpoint']);
        if(isset($form_values['miniorange_oauth_client_summary']['miniorange_auth_client_access_token_ep']))
            $access_token_ep = trim($form_values['miniorange_oauth_client_summary']['miniorange_auth_client_access_token_ep']['miniorange_auth_client_access_token_ep']);
        if(isset($form_values['miniorange_oauth_client_summary']['miniorange_auth_client_user_info_ep']))
            $user_info_ep = trim($form_values['miniorange_oauth_client_summary']['miniorange_auth_client_user_info_ep']['miniorange_auth_client_user_info_ep']);
        $email_attr = \Drupal::config('miniorange_oauth_client.settings')->get('miniorange_oauth_client_email_attr_val');
        if(($client_app=='Select') || empty($client_app) || empty($app_name) || empty($client_id) || empty($client_secret) || empty($authorize_endpoint) || empty($access_token_ep)
            || empty($user_info_ep)) {
            if(empty($client_app)|| $client_app == 'Select'){
                \Drupal::messenger()->addMessage(t('The <b>Select Application</b> dropdown is required. Please Select your application.'), 'error');
                return;
            }
            \Drupal::messenger()->addMessage(t('The <b>Display name</b>, <b>Client ID</b>, <b>Client Secret</b>, <b>Authorize Endpoint</b>, <b>Access Token Endpoint</b>
                , <b>Get User Info Endpoint</b> fields are required.'), 'error');
            return;
        }

        if(empty($client_app))
        {
            $client_app =\Drupal::config('miniorange_oauth_client.settings')->get('miniorange_oauth_client_app');
        }
        if(empty($app_name))
        {
            $client_app = \Drupal::config('miniorange_oauth_client.settings')->get('miniorange_auth_client_app_name');
        }
        if (empty($display_name))
        {
            $display_name = '';
        }
        if(empty($client_id))
        {
            $client_id =\Drupal::config('miniorange_oauth_client.settings')->get('miniorange_auth_client_client_id');
        }
        if(empty($client_secret))
        {
            $client_secret = \Drupal::config('miniorange_oauth_client.settings')->get('miniorange_auth_client_client_secret');
        }
        if(empty($scope))
        {
            $scope = \Drupal::config('miniorange_oauth_client.settings')->get('miniorange_auth_client_scope');
        }
        if(empty($authorize_endpoint))
        {
            $authorize_endpoint = \Drupal::config('miniorange_oauth_client.settings')->get('miniorange_auth_client_authorize_endpoint');
        }
        if(empty($access_token_ep))
        {
            $access_token_ep =\Drupal::config('miniorange_oauth_client.settings')->get('miniorange_auth_client_access_token_ep');
        }
        if(empty($user_info_ep))
        {
            $user_info_ep = \Drupal::config('miniorange_oauth_client.settings')->get('miniorange_auth_client_user_info_ep');
        }

        $callback_uri = $baseUrlValue."/mo_login";

        $app_values = \Drupal::config('miniorange_oauth_client.settings')->get('miniorange_oauth_client_appval');
        if(!is_array($app_values))
            $app_values = array();
        $app_values['client_id'] = $client_id;
        $app_values['client_secret'] = $client_secret;
        $app_values['app_name'] = $app_name;
        $app_values['display_name'] = $display_name;
        $app_values['scope'] = $scope;
        $app_values['authorize_endpoint'] = $authorize_endpoint;
        $app_values['access_token_ep'] = $access_token_ep;
        $app_values['user_info_ep'] = $user_info_ep;
        $app_values['callback_uri'] = $callback_uri;
        $app_values['client_app'] = $client_app;
        $app_values['miniorange_oauth_client_email_attr'] = $email_attr;
        $enable_login_with_oauth = $form_values['miniorange_oauth_client_summary']['miniorange_oauth_enable_login_with_oauth']['miniorange_oauth_enable_login_with_oauth'];
        $enable_login = $enable_login_with_oauth == 1 ? TRUE : FALSE;
        $enable_with_header = $form_values['miniorange_oauth_client_summary']['send_client_id_secret']['send_client_id_secret_oauth']['miniorange_oauth_send_with_header_oauth'];
        $enable_with_body = $form_values['miniorange_oauth_client_summary']['send_client_id_secret']['send_client_id_secret_oauth']['miniorange_oauth_send_with_body_oauth'];
        $enable_header = $enable_with_header == 1 ? TRUE : FALSE ;
        $enable_body = $enable_with_body == 1 ? TRUE : FALSE ;

        $config = \Drupal::configFactory()->getEditable('miniorange_oauth_client.settings');
        $config->set('miniorange_oauth_enable_login_with_oauth', $enable_login)
          ->set('miniorange_oauth_client_app', $client_app)
          ->set('miniorange_oauth_client_appval', $app_values)
          ->set('miniorange_auth_client_app_name', $app_name)
          ->set('miniorange_auth_client_display_name', $display_name)
          ->set('miniorange_auth_client_client_id', $client_id)
          ->set('miniorange_auth_client_client_secret', $client_secret)
          ->set('miniorange_auth_client_scope', $scope)
          ->set('miniorange_auth_client_authorize_endpoint', $authorize_endpoint)
          ->set('miniorange_auth_client_access_token_ep', $access_token_ep)
          ->set('miniorange_auth_client_user_info_ep', $user_info_ep)
          ->set('miniorange_auth_client_stat', "Review Config")
          ->set('miniorange_auth_client_callback_uri', $callback_uri)
          ->set('miniorange_oauth_send_with_header_oauth', $enable_header)
          ->set('miniorange_oauth_send_with_body_oauth', $enable_body)
          ->save();
        \Drupal::messenger()->addMessage(t('Configurations saved successfully. Please click on the <b>Test Configuration</b> button to test the connection.'), 'status');
        if(isset($_GET['action']) && $_GET['action'] == 'update'){
            $response = new RedirectResponse($base_url.'/admin/config/people/miniorange_oauth_client/config_clc');
            $response->send();
            return new Response();
        }
    }

}
