<?php

namespace Drupal\miniorange_oauth_client\Form;

use Drupal;

use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Config\Config;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\miniorange_oauth_client\Utilities;
use Drupal\miniorange_oauth_client\Controller\miniorange_oauth_clientController;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\RedirectResponse;


class MoOAuthBackupConfig extends FormBase
{

    private ImmutableConfig $config;
    private Config $config_factory;

    public function __construct()
    {
        $this->config = Drupal::config('miniorange_oauth_client.settings');
        $this->config_factory = \Drupal::configFactory()->getEditable('miniorange_oauth_client.settings');
    }

    /**
     * @inheritDoc
     */
    public function getFormId()
    {
        return 'mo_oauth_client_backup_config';
    }

    /**
     * @inheritDoc
    */
    public function buildForm(array $form, FormStateInterface $form_state)
    {
        global $base_url;
        $form['#prefix'] = '<div id="modal_backup_form">';
        $form['#suffix'] = '</div>';
        $form['status_messages'] = [
            '#type' => 'status_messages',
            '#weight' => -10,
        ];

        $form['button'] = array(
            '#type' => 'submit',
            '#button_type' => 'danger',
            '#value' => t('&#11164; Back'),
            '#submit' => array('::back_to_config_table')
        );

        $form['markup_top_vt_start2'] = array(
            '#type' => 'fieldset',
            '#title' => $this->t('BACKUP/IMPORT CONFIGURATIONS'),
            '#attributes' => array( 'style' => 'padding:2% 2% 5%; margin-bottom:2%' ),
        );

        $module_path = \Drupal::service('extension.list.module')->getPath('miniorange_oauth_client');

        $form['markup_top_vt_start2']['markup_1'] = array(
            '#markup' => '<br><div class="mo_oauth_highlight_background_note"><p><b>NOTE: </b>This tab will help you to transfer your module configurations when you change your Drupal instance.
                          <br>Example: When you switch from test environment to production.<br>Follow these 3 simple steps to do that:<br>
                          <br><strong>1.</strong> Download module configuration file by clicking on the Download Configuration button given below.
                          <br><strong>2.</strong> Install the module on new Drupal instance.<br><strong>3.</strong> Upload the configuration file in Import module Configurations section.<br>
                          <br><b>And just like that, all your module configurations will be transferred!</b></p></div><br><div id="Exort_Configuration"><h3>Backup/ Export Configuration &nbsp;&nbsp;</h3><hr/><p>
                          Click on the button below to download module configuration.</p>',
        );

        $form['markup_top_vt_start2']['miniorange_saml_imo_option_exists_export'] = array(
            '#type' => 'submit',
            '#button_type' => 'primary',
            '#value' => $this->t('Download'),
            '#submit' => array('::miniorange_import_export'),
            '#suffix'=> '<br/><br/></div>',
        );

        $form['markup_top_vt_start2']['markup_prem_plan'] = array(
            '#markup' => '<div id="Import_Configuration"><br/><h3>Import Configuration</h3><hr><br>
                          <div class="mo_oauth_highlight_background_note_1"><b>Note: </b>Available in
                          <a href="' . $base_url . '/admin/config/people/miniorange_oauth_client/licensing">Standard, Premium and Enterprise</a> versions of the module</div>',
        );

        $form['markup_top_vt_start2']['markup_import_note'] = array(
            '#markup' => '<p>This tab will help you to<span style="font-weight: bold"> Import your module configurations</span> when you change your Drupal instance.</p>
                 <p>choose <b>"json"</b> Extened module configuration file and upload by clicking on the button given below. </p>',
        );

        $form['markup_top_vt_start2']['import_Config_file'] = array(
            '#type' => 'file',
            '#disabled' => TRUE,
        );

        $form['markup_top_vt_start2']['miniorange_saml_idp_import'] = array(
            '#type' => 'submit',
            '#value' => t('Upload'),
            '#disabled' => TRUE,
        );


        return $form;
    }


    public function miniorange_import_export(array &$form, FormStateInterface $form_state)
	{
        $tab_class_name = array(
            'OAuth Client Configuration' => 'mo_options_enum_client_configuration',
            'Attribute Mapping' => 'mo_options_enum_attribute_mapping',
            'Sign In Settings' => 'mo_options_enum_signin_settings'
        );

        $configuration_array = array();
        foreach($tab_class_name as $key => $value) {
            $configuration_array[$key] = $this->mo_get_configuration_array($value);
        }

        $configuration_array["Version_dependencies"] = $this->mo_get_version_informations();
        $this->config_factory->set('miniorange_oauth_client_module_configuration',$configuration_array)->save();
        header("Content-Disposition: attachment; filename = miniorange_oauth_client_config.json");
        echo(json_encode($configuration_array, JSON_PRETTY_PRINT|JSON_UNESCAPED_SLASHES));
        exit;
	}

    public function back_to_config_table(){
        $response = new RedirectResponse(Url::fromRoute('miniorange_oauth_client.config_clc')->toString());
        $response->send();
        return new Response();
    }

    public function mo_get_configuration_array($class_name)
    {
        $class_object = Utilities::getVariableArray($class_name);
        $mo_array = array();
        foreach($class_object as $key => $value) {
            $mo_option_exists = $this->config->get($value);
            if($mo_option_exists) {
                $mo_array[$key] = $mo_option_exists;
            }
        }
        return $mo_array;
    }

    public function mo_get_version_informations() {
        $array_version = array();
        $array_version["PHP_version"] = phpversion();
        $array_version["Drupal_version"] = \DRUPAL::VERSION;
        $array_version["OPEN_SSL"] = $this->mo_oauth_is_openssl_installed();
        $array_version["CURL"] = $this->mo_oauth_is_curl_installed();
        $array_version["ICONV"] = $this->mo_oauth_is_iconv_installed();
        $array_version["DOM"] = $this->mo_oauth_is_dom_installed();
        return $array_version;
    }

    public function mo_oauth_is_openssl_installed() {
		if ( in_array( 'openssl', get_loaded_extensions() ) ) {
			return 1;
		} else {
			return 0;
		}
	}

    public function mo_oauth_is_curl_installed() {
        if ( in_array( 'curl', get_loaded_extensions() ) ) {
            return 1;
        } else {
            return 0;
        }
    }

    public function mo_oauth_is_iconv_installed() {
        if ( in_array( 'iconv', get_loaded_extensions() ) ) {
            return 1;
        } else {
            return 0;
        }
    }

    public function mo_oauth_is_dom_installed() {
        if ( in_array( 'dom', get_loaded_extensions() ) ) {
            return 1;
        } else {
            return 0;
        }
    }

    public function validateForm(array &$form, FormStateInterface $form_state) { }

    public function submitForm(array &$form, FormStateInterface $form_state) { }
}