<?php

namespace Drupal\miniorange_oauth_client\Form;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\FormBase;
use Drupal\miniorange_oauth_client\Utilities;
use Symfony\Component\HttpFoundation\RedirectResponse;

class Settings extends FormBase
{
    public function getFormId() {
        return 'miniorange_oauth_client_settings';
    }
    /**
     * Showing Settings form.
     */
    public function buildForm(array $form, FormStateInterface $form_state) {
        global $base_url;
        $url_path = $base_url . '/' . \Drupal::service('extension.list.module')->getPath('miniorange_oauth_client'). '/includes/images';
        $baseUrlValue = \Drupal::config('miniorange_oauth_client.settings')->get('miniorange_oauth_client_base_url');

        $attachments['#attached']['library'][] = 'miniorange_oauth_client/miniorange_oauth_client.admin';

        $form['markup_library'] = array(
            '#attached' => array(
                'library' => array(
                    "miniorange_oauth_client/miniorange_oauth_client.admin",
                    "miniorange_oauth_client/miniorange_oauth_client.style_settings",
                )
            ),
        );

        $form['header_top_style_1'] = array('#markup' => '<div class="mo_oauth_table_layout_1">');

        $form['markup_top'] = array(
            '#markup' => '<div class="mo_oauth_table_layout mo_oauth_container">',
        );

        $form['markup_custom_role_mapping'] = array(
            '#type' => 'fieldset',
             '#title' => t('Sign In Settings'),
            '#markup' => '<hr>',
        );

        $form['markup_custom_role_mapping']['base_site_url_note'] = array(
            '#markup' => '<p>If your provider only support HTTPS Callback URL and you have HTTP site, just save your base site URL with HTTPS here.</p>'
        );

        $module_path = \Drupal::service('extension.list.module')->getPath('miniorange_oauth_client');

        $form['markup_custom_role_mapping']['markup_top'] = array(
            '#markup' => '<div class="container-inline">',
        );


        $form['markup_custom_role_mapping']['miniorange_oauth_client_base_url_title'] = array(
            '#markup' => '<b>Base/Site URL:</b> <div class="mo_oauth_tooltip"><img src="'.$base_url.'/'. $module_path . '/includes/images/info.png" alt="info icon" height="15px" width="15px"></div><div class="mo_oauth_tooltiptext"><b>Note: </b>You can customize base URL here. (For eg: https://www.xyz.com or http://localhost/abc)</div>',
        );

        $form['markup_custom_role_mapping']['miniorange_oauth_client_base_url'] = array(
            '#type' => 'textfield',
            '#default_value' => $baseUrlValue,
            '#attributes' => array('id'=>'mo_oauth_vt_baseurl','style' => 'margin-left: 12%; width:80%;','placeholder' => 'Enter Base URL'),
        );

        $form['markup_custom_role_mapping']['miniorange_oauth_client_siginin1'] = array(
            '#type' => 'submit',
            '#button_type' => 'primary',
             '#attributes' => array('style' => 'margin-left: 12%;'),
            '#value' => t('Update'),
        );

        $form['markup_custom_role_mapping']['markup_top_end_div'] = array(
            '#markup' => '</div>',
        );

        $form['markup_custom_troubleshoot'] = array(
            '#type' => 'fieldset',
            '#title' => t('Debugging & Troubleshoot'),
        );

       $form['markup_custom_troubleshoot']['miniorange_oauth_client_enable_logging'] = array(
         '#type' => 'checkbox',
         '#title' => t('Enable Logging'),
         '#default_value' => \Drupal::config('miniorange_oauth_client.settings')->get('miniorange_oauth_client_enable_logging'),
         '#description' => 'Enabling this checkbox will add loggers under the <a href="'.$base_url.'/admin/reports/dblog?type%5B%5D=miniorange_oauth_client" target="_blank">Reports</a> section',
         '#prefix' => '<hr>',
       );

       $form['markup_custom_troubleshoot']['miniorange_oauth_client_siginin1'] = array(
         '#type' => 'submit',
         '#button_type' => 'primary',
         '#value' => t('Save Configuration'),
         '#submit' => array('::TroubleshootsubmitForm'),

       );

       $form['markup_custom_troubleshoot']['miniorange_oauth_client_enable_logging_download'] = array(
        '#type' => 'submit',
        '#value' => t('Download Module Logs'),
        '#limit_validation_errors' => array(),
        '#states' => [
            'disabled' => [
                ':input[name="miniorange_oauth_client_enable_logging"]' => ['checked' =>false],
            ],
        ],
        '#submit' => array('::miniorange_module_logs'),
      );

        $form['markup_custom_auto_create_user'] = array(
            '#type' => 'fieldset',
            '#title' => t('Auto Create Users <a href="licensing"><img class="mo_oauth_pro_icon1" src="' . $url_path . '/pro.png" alt="Premium and Enterprise"><span class="mo_pro_tooltip">Available in the Standard, Premium and Enterprise version</span></a>'),
            '#markup' => '<hr>',
        );

        $form['markup_custom_auto_create_user']['markup_bottom_vt_start_auto_create_users'] = array(
            '#markup' => '<p>This feature provides you with an option to automatically create a user if the user is not already present in Drupal</p>'
        );

        $form['markup_custom_auto_create_user']['miniorange_oauth_disable_autocreate_users'] = array(
            '#type' => 'checkbox',
            '#title' => t('Check this option if you want to enable <b>auto creation</b> of users if user does not exist.'),
            '#disabled' => TRUE,
        );

        $form['markup_custom_signin'] = array(
            '#type' => 'fieldset',
            '#title' => t('Advanced Sign In Settings <a href="licensing"><img class="mo_oauth_pro_icon1" src="' . $url_path . '/pro.png" alt="Premium and Enterprise"><span class="mo_pro_tooltip">Available in the Premium and Enterprise version</span></a><a class="mo_oauth_client_how_to_setup" href="https://developers.miniorange.com/docs/oauth-drupal/sign-in-settings#sign-in-settings-features" target="_blank">[What is Force Authentication and How to Set up]</a>'),
            '#markup' => '<hr><br>',
        );

        $form['markup_custom_signin']['miniorange_oauth_force_auth'] = array(
            '#type' => 'checkbox',
            '#title' => t('Protect website against anonymous access '),
            '#disabled' => TRUE,
            '#description' => t('<b>Note: </b>Users will be redirected to your OAuth server for login in case user is not logged in and tries to access website.<br><br>'),
        );

        $form['markup_custom_signin']['miniorange_oauth_auto_redirect'] = array(
            '#type' => 'checkbox',
            '#title' => t('Check this option if you want to <b> Auto-redirect to OAuth Provider/Server </b>'),
            '#disabled' => TRUE,
            '#description' => t('<b>Note: </b>Users will be redirected to your OAuth server for login when the login page is accessed.<br><br>'),
        );

        $form['markup_custom_signin']['miniorange_oauth_enable_backdoor'] = array(
            '#type' => 'checkbox',
            '#title' => t('Check this option if you want to enable <b>backdoor login </b>'),
            '#disabled' => TRUE,
            '#description' => t('<b>Note:</b> Checking this option creates a backdoor to login to your Website using Drupal credentials<br> incase you get locked out of your OAuth server.<br>'),
        );

        $form['markup_custom_signin1'] = array(
            '#type' => 'fieldset',
            '#title' => t('Domain & Page Restriction <a href="licensing"><img class="mo_oauth_pro_icon1" src="' . $url_path . '/pro.png" alt="Premium and Enterprise"><span class="mo_pro_tooltip">Available in the Enterprise version</span></a><a class="mo_oauth_client_how_to_setup" href="https://developers.miniorange.com/docs/oauth-drupal/sign-in-settings#domain-restriction" target="_blank">[What is Domain/Page restriction and How to Set up]</a>'),
            '#markup' => '<hr><br>',
        );

        $form['markup_custom_signin1']['miniorange_oauth_client_white_list_url'] = array(
            '#type' => 'textfield',
            '#title' => t('Allowed Domains'),
            '#attributes' => array('style' => 'width:80%','placeholder' => 'Enter semicolon(;) separated domains (Eg. xxxx.com; xxxx.com)'),
            '#disabled' => TRUE,
            '#description' => t("<b>Note:</b> In this feature, you can allow only some of the domains to login using the SSO."),
        );

        $form['markup_custom_signin1']['miniorange_oauth_client_black_list_url'] = array(
            '#type' => 'textfield',
            '#title' => t('Restricted Domains'),
            '#attributes' => array('style' => 'width:80%','placeholder' => 'Enter semicolon(;) separated domains (Eg. xxxx.com; xxxx.com)'),
            '#disabled' => TRUE,
            '#description' => t("<b>Note:</b> In this feature, you can restrict some of the domains to login using the SSO."),
        );

        $form['markup_custom_signin1']['miniorange_oauth_client_page_restrict_url'] = array(
            '#type' => 'textfield',
            '#title' => t('Page Restriction'),
            '#attributes' => array('style' => 'width:80%','placeholder' => 'Enter semicolon(;) separated page URLs (Eg. xxxx.com/yyy; xxxx.com/yyy)'),
            '#disabled' => TRUE,
            '#description' => t("<b>Note:</b> In this feature, you can restrict unauthorized access for some of the URLs of the site."),
            '#suffix' => '<br>',
        );


      $form['markup_custom_login_logout'] = array(
          '#type' => 'fieldset',
          '#title' => t('Custom Login/Logout <a href="licensing"><img class="mo_oauth_pro_icon1" src="' . $url_path . '/pro.png" alt="Premium and Enterprise"><span class="mo_pro_tooltip">Available in the Standard, Premium and Enterprise version</span></a>'),

      );

      $form['markup_custom_login_logout']['custom_redirect'] = array(
        '#markup' => '<p>This feature provides you with an option to redirect the users to a default page after Login and Logout</p>',
        '#prefix' => '<hr>',
       );

      $form['markup_custom_login_logout']['miniorange_oauth_client_login_url'] = array(
          '#type' => 'textfield',
          '#disabled' => TRUE,
          '#title' => t('Default redirect URL after login'),
          '#attributes' => array('style' => 'width:73%;background-color: hsla(0,0%,0%,0.08) !important;','placeholder' => 'Enter Login URL'),
      );

      $form['markup_custom_login_logout']['miniorange_oauth_client_logout_url'] = array(
          '#type' => 'textfield',
          '#disabled' => TRUE,
          '#title' => t('Default redirect URL after logout'),
          '#attributes' => array('style' => 'width:73%;background-color: hsla(0,0%,0%,0.08) !important;','placeholder' => 'Enter Logout URL'),
      );

        $form['markup_custom_login_button'] = array(
            '#type' => 'fieldset',
            '#title' => t('Login Button Customization <a href="licensing"><img class="mo_oauth_pro_icon1" src="' . $url_path . '/pro.png" alt="Premium and Enterprise"><span class="mo_pro_tooltip">Available in the Standard, Premium and Enterprise version</span></a>'),

            '#markup' => '<hr><br>',
        );

        $form['markup_custom_login_button']['miniorange_oauth_icon_width'] = array(
            '#type' => 'textfield',
            '#title' => t('Icon width'),
            '#disabled' => TRUE,
            '#description' => t('For eg. 200px or 10% <br>'),
        );

        $form['markup_custom_login_button']['miniorange_oauth_icon_height'] = array(
            '#type' => 'textfield',
            '#title' => t('Icon height'),
            '#disabled' => TRUE,
            '#description' => t('For eg. 60px or auto <br>'),
        );

        $form['markup_custom_login_button']['miniorange_oauth_icon_margins'] = array(
            '#type' => 'textfield',
            '#title' => t('Icon Margins'),
            '#disabled' => TRUE,
            '#description' => t('For eg. 2px 3px or auto <br>'),
        );

        $form['markup_custom_login_button']['miniorange_oauth_custom_css'] = array(
            '#type' => 'textarea',
            '#title' => t('Custom CSS'),
            '#disabled' => TRUE,
            '#attributes' => array('style'=> 'width:80%', 'placeholder' => 'For eg.  .oauthloginbutton{ background: #7272dc; height:40px; padding:8px; text-align:center; color:#fff; }'),
        );

        $form['markup_custom_login_button']['miniorange_oauth_btn_txt'] = array(
            '#type' => 'textfield',
            '#title' => t('Custom Button Text'),
            '#disabled' => TRUE,
            '#attributes' => array('placeholder'=> 'Login Using appname'),
        );
        $form['miniorange_oauth_client_siginin'] = array(

            '#type' => 'button',
            '#value' => t('Save Configuration'),
            '#button_type' => 'primary',
            '#disabled' => TRUE,
            '#attributes' => array('style' => '	margin: auto; display:block; '),
        );

        $form['mo_header_style_end'] = array('#markup' => '</div>');
        Utilities::moOAuthShowCustomerSupportIcon($form, $form_state);
        return $form;
    }

    public function submitForm(array &$form, FormStateInterface $form_state) {
        global $base_url;

        $baseUrlvalue = trim($form['markup_custom_role_mapping']['miniorange_oauth_client_base_url']['#value']);
        if(!empty($baseUrlvalue) && filter_var($baseUrlvalue, FILTER_VALIDATE_URL) == FALSE) {
            \Drupal::messenger()->adderror(t('Please enter a valid URL'));
            return;
        }
        $callbackUrl = empty($baseUrlvalue)?$base_url."/mo_login":$baseUrlvalue."/mo_login";

        $configFactory=\Drupal::configFactory()->getEditable('miniorange_oauth_client.settings');
        $configFactory->set('miniorange_oauth_client_base_url', $baseUrlvalue)
                      ->set('miniorange_auth_client_callback_uri', $callbackUrl)
                      ->save();
        \Drupal::messenger()->addMessage(t('Configurations saved successfully.'));
    }

    public function TroubleshootsubmitForm(array &$form, FormStateInterface $form_state) {
     $form_values = $form_state->getValues();
     $enable_logs = $form_values['miniorange_oauth_client_enable_logging'];
     \Drupal::configFactory()->getEditable('miniorange_oauth_client.settings')->set('miniorange_oauth_client_enable_logging',$enable_logs)->save();
     \Drupal::messenger()->addMessage(t('Configurations saved successfully.'));
   }


 public static function mofilterData(&$str){
    $str = preg_replace("/\t/", "\\t", $str);
    $str = preg_replace("/\r?\n/", "\\n", $str);
    if(strstr($str, '"')) $str = '"' . str_replace('"', '""', $str) . '"';
  }

  public static function miniorange_module_logs(){

    $connection = \Drupal::database();

// Excel file name for download
    $fileName = "drupal_oauth_client_loggers_" . date('Y-m-d') . ".xls";

// Column names
    $fields = array('WID', 'UID', 'TYPE', 'MESSAGE', 'VARIABLES', 'SEVERITY', 'LINK', 'LOCATION', 'REFERER', 'HOSTNAME', 'TIMESTAMP');

// Display column names as first row
    $excelData = implode("\t", array_values($fields)) . "\n\n";

// Fetch records from database
    $query = $connection->query("SELECT * from {watchdog} WHERE type = 'miniorange_oauth_client' OR severity = 3")->fetchAll();

    foreach ($query as $row){
      $lineData = array($row->wid, $row->uid, $row->type, $row->message, $row->variables, $row->severity, $row->link, $row->location, $row->referer, $row->hostname, $row->timestamp);
      array_walk($lineData, array('self','mofilterData'));
      $excelData .= implode("\t", array_values($lineData)) . "\n";
    }

// Headers for download
    header("Content-Type: application/vnd.ms-excel");
    header("Content-Disposition: attachment; filename=\"$fileName\"");

// Render excel data
    echo $excelData;
    exit;
  }

}
