<?php
/**
 * @file
 * Contains miniOrange Customer class.
 */

/**
 * @file
 * This class represents configuration for customer.
 */
namespace Drupal\miniorange_oauth_client;
use Drupal\miniorange_oauth_client\Utilities;
class MiniorangeOAuthClientCustomer {

  public $email;

  public $phone;

  public $customerKey;

  public $transactionId;

  public $password;

  public $otpToken;

  private $defaultCustomerId;

  private $defaultCustomerApiKey;

  /**
   * Constructor.
   */
  public function __construct($email, $phone, $password, $otp_token) {
    $this->email = $email;
    $this->phone = $phone;
    $this->password = $password;
    $this->otpToken = $otp_token;
    $this->defaultCustomerId = "16555";
    $this->defaultCustomerApiKey = "fFd2XcvTGDemZvbw1bcUesNJWEqKbbUq";
  }

  /**
   * Check if customer exists.
   */
  public function checkCustomer() {
    if (!Utilities::isCurlInstalled()) {
      return json_encode(array(
        "status" => 'CURL_ERROR',
        "statusMessage" => '<a href="http://php.net/manual/en/curl.installation.php">PHP cURL extension</a> is not installed or disabled.',
      ));
    }

    $url = MiniorangeOAuthClientConstants::BASE_URL . '/moas/rest/customer/check-if-exists';
   
    $email = $this->email;

    $header = [
      'Content-Type' => 'application/json', 'charset'=> 'UTF - 8',
      'Authorization' => 'Basic',
    ];

    $body= [
      'email' => $email,
    ];

    $fields = [
      'headers' => $header,'body' => json_encode($body),'verify' => false,
    ];    
   
    try{

      $api_response = \Drupal::httpClient()->request('POST', $url , $fields);

      $response = $api_response->getBody()->getContents();

      return $response;
     }catch (RequestException $exception){
      $error = array('%method' => 'checkCustomer',
        '%file' => 'customer_setup.php',
        '%error' => $exception->getMessage()
      );
      \Drupal::logger('miniorange_oauth_client')->notice($error);
    }

  }

  /**
   * Create Customer.
   */
  public function createCustomer() {
    if (!Utilities::isCurlInstalled()) {
      return json_encode(array(
        "statusCode" => 'ERROR',
        "statusMessage" => '. Please check your configuration.',
      ));
    }
    $url = MiniorangeOAuthClientConstants::BASE_URL . '/moas/rest/customer/add';

    $header = [
      'Content-Type' => 'application/json', 'charset'=> 'UTF - 8',
      'Authorization' => 'Basic',
    ];

    $body= [
      'companyName' => $_SERVER['SERVER_NAME'],
      'areaOfInterest' => 'DRUPAL ' . Utilities::mo_get_drupal_core_version() . ' OAuth Client',
      'email' => $this->email,
      'phone' => $this->phone,
      'password' => $this->password,
    ];

    $fields = [
      'headers' => $header,'body' => json_encode($body),'verify' => false,
    ];
    
    try{
      $api_response = \Drupal::httpClient()->request('POST', $url , $fields);

      $response = $api_response->getBody()->getContents();

      return $response;
     }catch (RequestException $exception){
      $error = array('%method' => 'createCustomer',
        '%file' => 'customer_setup.php',
        '%error' => $exception->getMessage()
      );
      \Drupal::logger('miniorange_oauth_client')->notice($error);
    }

  }

  /**
   * Get Customer Keys.
   */
  public function getCustomerKeys() {
    if (!Utilities::isCurlInstalled()) {
      return json_encode(array(
        "apiKey" => 'CURL_ERROR',
        "token" => '<a href="http://php.net/manual/en/curl.installation.php">PHP cURL extension</a> is not installed or disabled.',
      ));
    }

    $url = MiniorangeOAuthClientConstants::BASE_URL . '/moas/rest/customer/key';

    $email = $this->email;
    $password = $this->password;

    $header = [
      'Content-Type' => 'application/json', 'charset'=> 'UTF - 8', 'Authorization' => 'Basic',
    ];

    $body = [
      'email' => $email, 'password' => $password,
    ];

    $fields = [
      'headers' => $header, 'body' => json_encode($body), 'verify' => false,
    ];


    try{

      $api_response = \Drupal::httpClient()->request('POST', $url , $fields);
      $response = $api_response->getBody()->getContents();

      return $response;
    }catch (RequestException $exception){
      $error = array(
        '%method' => 'getCustomerKeys',
        '%file' => 'customer_setup.php',
        '%error' => $exception->getMessage(),
      );
      \Drupal::logger('miniorange_oauth_client')->notice($error);
    }

  }

  /**
   * Send OTP.
  */
  public function sendOtp() {
    if (!Utilities::isCurlInstalled()) {
      return json_encode(array(
        "status" => 'CURL_ERROR',
        "statusMessage" => '<a href="http://php.net/manual/en/curl.installation.php">PHP cURL extension</a> is not installed or disabled.',
      ));
    }
    $url = MiniorangeOAuthClientConstants::BASE_URL . '/moas/api/auth/challenge';
    $customer_key = $this->defaultCustomerId;
    $api_key = $this->defaultCustomerApiKey;

    $username = \Drupal::config('miniorange_oauth_client.settings')->get('miniorange_oauth_client_customer_admin_email');

    /* Current time in milliseconds since midnight, January 1, 1970 UTC. */
    $currentTimeInMillis = round(microtime(TRUE) * 1000);

    /* Creating the Hash using SHA-512 algorithm */
    $string_to_hash = $customer_key . number_format($currentTimeInMillis, 0, '', '' ) . $api_key;
    $hash_value = hash("sha512", $string_to_hash);

    $customer_key_header = "Customer-Key: " . $customer_key;
    $timestamp_header = "Timestamp: " . number_format($currentTimeInMillis, 0, '', '' );
    $authorization_header = "Authorization: " . $hash_value;

    $header = [
      "Content-Type" =>"application/json", "Customer-Key" => $customer_key ,"Timestamp" => $currentTimeInMillis, "Authorization" => $hash_value,
    ];

    $body= [
      'customerKey' => $customer_key,
      'email' => $username,
      'authType' => 'EMAIL',
      'transactionName' => 'Drupal ' . Utilities::mo_get_drupal_core_version() . ' OAuth Client Module',
    ];

    $fields = [
      'headers' => $header,'body' => json_encode($body),'verify' => false,
    ];
    
    try{

      $api_response = \Drupal::httpClient()->request('POST', $url , $fields);

      $response = $api_response->getBody()->getContents();

      return $response;
     }catch (RequestException $exception){
      $error = array('%method' => 'sendotp',
        '%file' => 'customer_setup.php',
        '%error' => $exception->getMessage()
      );
      \Drupal::logger('miniorange_oauth_client')->notice($error);
    }
  }

  /**
   * Validate OTP.
   */
  public function validateOtp($transaction_id) {
    if (!Utilities::isCurlInstalled()) {
      return json_encode(array(
        "status" => 'CURL_ERROR',
        "statusMessage" => '<a href="http://php.net/manual/en/curl.installation.php">PHP cURL extension</a> is not installed or disabled.',
      ));
    }

    $url = MiniorangeOAuthClientConstants::BASE_URL . '/moas/api/auth/validate';

    $customer_key = $this->defaultCustomerId;
    $api_key = $this->defaultCustomerApiKey;

    $currentTimeInMillis = round(microtime(TRUE) * 1000);

    $string_to_hash = $customer_key . number_format($currentTimeInMillis, 0, '', '' ) . $api_key;
    $hash_value = hash("sha512", $string_to_hash);

    $customer_key_header = "Customer-Key: " . $customer_key;
    $timestamp_header = "Timestamp: " . number_format($currentTimeInMillis, 0, '', '' );
    $authorization_header = "Authorization: " . $hash_value;

    $header = [
      "Content-Type" =>"application/json", "Customer-Key" => $customer_key ,"Timestamp" => $currentTimeInMillis, "Authorization" => $hash_value,
    ];

    $body= [
      'txId' => $transaction_id,
      'token' => $this->otpToken,
    ];

    $fields = [
      'headers' => $header,'body' => json_encode($body),'verify' => false,
    ];

    try{

      $api_response = \Drupal::httpClient()->request('POST', $url , $fields);

      $response = $api_response->getBody()->getContents();

      return $response;
     }catch (RequestException $exception){
      $error = array('%method' => 'validateotp',
        '%file' => 'customer_setup.php',
        '%error' => $exception->getMessage()
      );
      \Drupal::logger('miniorange_oauth_client')->notice($error);
    }

  }

}
